import datetime

from pyramid.httpexceptions import HTTPForbidden
from pyramid.view import view_config
from sqlalchemy import func

from barker.models import Inventory, Kot, Product, MenuCategory, Settlement, SettleOption, Tax, Voucher
import datetime

from pyramid.httpexceptions import HTTPForbidden
from pyramid.view import view_config
from sqlalchemy import func

from barker.models import Inventory, Kot, Product, MenuCategory, Settlement, SettleOption, Tax, Voucher


@view_config(request_method='GET', route_name='sa_sale', renderer='json', permission='Sales Analysis',
             request_param=('s', 'f'))
def get_sale(request):
    start_date = datetime.datetime.strptime(request.GET['s'], '%d-%b-%Y %H:%M')
    finish_date = datetime.datetime.strptime(request.GET['f'], '%d-%b-%Y %H:%M')

    if (datetime.date.today() - start_date.date()).days > 5 and 'Accounts Audit' not in request.effective_principals:
        raise HTTPForbidden("Accounts Audit")

    amount = func.sum(Inventory.quantity * Inventory.effective_price * (1 - Inventory.discount)).label('Amount')
    list = request.dbsession.query(
        MenuCategory.group_type, amount
    ).join(Voucher.kots).join(Kot.inventories).join(Inventory.product).join(Product.menu_category).filter(
        Voucher.date >= start_date,
        Voucher.date <= finish_date,
        Voucher.is_void == False,
        Voucher.settlements.any(~Settlement.settled.in_([1, 4, 7, 8, 9, 10]))
    ).group_by(
        MenuCategory.group_type
    ).order_by(
        MenuCategory.group_type
    ).all()
    total = 0
    info = []
    for gt, am in list:
        total += am
        info.append({'GroupType': gt, 'Amount': am})
    return info + [{'GroupType': 'Total Settled', 'Amount': total}]


@view_config(request_method='GET', route_name='sa_settlements', renderer='json', permission='Sales Analysis',
             request_param=('s', 'f'))
def get_settlements(request):
    start_date = datetime.datetime.strptime(request.GET['s'], '%d-%b-%Y %H:%M')
    finish_date = datetime.datetime.strptime(request.GET['f'], '%d-%b-%Y %H:%M')

    if (datetime.date.today() - start_date.date()).days > 5 and 'Accounts Audit' not in request.effective_principals:
        raise HTTPForbidden("Accounts Audit")

    list = request.dbsession.query(
        SettleOption.name, func.sum(Settlement.amount)
    ).join(Voucher.settlements).join(Settlement.settle_option).filter(
        Voucher.date >= start_date,
        Voucher.date <= finish_date,
        Voucher.is_void == False
    ).group_by(
        SettleOption.name
    ).order_by(
        SettleOption.name
    ).all()
    total = 0
    info = []
    for gt, am in list:
        total += am
        info.append({'GroupType': gt, 'Amount': am})
    return info + [{'GroupType': 'Total', 'Amount': total}]


@view_config(request_method='GET', route_name='sa_tax', renderer='json', permission=('Tax Analysis', 'Sales Analysis'),
             request_param=('s', 'f'))
def get_tax(request):
    start_date = datetime.datetime.strptime(request.GET['s'], '%d-%b-%Y %H:%M')
    finish_date = datetime.datetime.strptime(request.GET['f'], '%d-%b-%Y %H:%M')

    if (datetime.date.today() - start_date.date()).days > 5 and 'Accounts Audit' not in request.effective_principals:
        raise HTTPForbidden("Accounts Audit")

    amounts = request.dbsession.query(
        Tax.name,
        Inventory.tax_rate,
        func.coalesce(func.sum(Inventory.net_taxable), 0),
        func.coalesce(func.sum(Inventory.tax_amount), 0)
    ).join(Voucher.kots).join(Kot.inventories).join(Inventory.tax).filter(
        Voucher.date >= start_date,
        Voucher.date <= finish_date,
        Voucher.is_void == False,
        Voucher.settlements.any(~Settlement.settled.in_([1, 4, 7, 8, 9, 10]))
    ).group_by(
        Tax.name,
        Inventory.tax_rate
    ).order_by(
        Tax.name,
        Inventory.tax_rate
    ).all()
    return [{
                'Name': "{0} - {1:.2%}".format(i[0], i[1]),
                'TaxRate': i[1],
                'NetSale': i[2],
                'TaxAmount': i[3]
            } for i in amounts]
